import React, { Fragment } from 'react'
import { get } from 'lodash'
import { H4 } from '@pubsweet/ui'
import styled from 'styled-components'
import { th } from '@pubsweet/ui-toolkit'
import { withProps, compose } from 'recompose'

import {
  Tag,
  Text,
  Tabs,
  marginHelper,
  ContextualBox,
  ReviewersTable,
  ReviewerReport,
  InviteReviewers,
  ReviewerBreakdown,
  withFilePreview,
  withFileDownload,
} from '../'

const ReviewerDetails = ({
  journal,
  reports,
  fragment,
  invitations,
  previewFile,
  downloadFile,
  canInviteReviewers,
  onInviteReviewer,
  onResendReviewerInvite,
  onRevokeReviewerInvite,
  canViewReviewersDetails,
}) =>
  canViewReviewersDetails ? (
    <ContextualBox
      label="Reviewer Details & Reports"
      rightChildren={
        <ReviewerBreakdown fitContent fragment={fragment} mr={1} />
      }
      startExpanded
    >
      <Tabs>
        {({ selectedTab, changeTab }) => (
          <Fragment>
            <TabsHeader>
              <TabButton
                ml={1}
                mr={1}
                onClick={() => changeTab(0)}
                selected={selectedTab === 0}
              >
                <H4>Reviewer Details</H4>
              </TabButton>

              <TabButton
                ml={1}
                mr={1}
                onClick={() => changeTab(1)}
                selected={selectedTab === 1}
              >
                <H4>Reviewer Reports</H4>
                <Tag mr={1}>{reports.length}</Tag>
              </TabButton>
            </TabsHeader>
            <TabContent>
              {selectedTab === 0 && (
                <Fragment>
                  {canInviteReviewers && (
                    <InviteReviewers
                      modalKey="invite-reviewers"
                      onInvite={onInviteReviewer}
                    />
                  )}
                  <ReviewersTable
                    invitations={invitations}
                    onResendReviewerInvite={onResendReviewerInvite}
                    onRevokeReviewerInvite={onRevokeReviewerInvite}
                  />
                </Fragment>
              )}
              {selectedTab === 1 && (
                <Fragment>
                  {reports.length === 0 && (
                    <Text align="center">No reports submitted yet.</Text>
                  )}
                  {reports.map((report, index) => (
                    <ReviewerReport
                      journal={journal}
                      key={report.id}
                      onDownload={downloadFile}
                      onPreview={previewFile}
                      report={report}
                      reviewerIndex={index + 1}
                      showOwner
                    />
                  ))}
                </Fragment>
              )}
            </TabContent>
          </Fragment>
        )}
      </Tabs>
    </ContextualBox>
  ) : null

export default compose(
  withFilePreview,
  withFileDownload,
  withProps(({ invitations = [], reviewerReports = [], currentUser }) => ({
    token: get(currentUser, 'token', ''),
    invitations: invitations.map(i => ({
      ...i,
      review: reviewerReports.find(r => r.userId === i.userId),
    })),
    reports: reviewerReports.filter(r => r.submittedOn),
  })),
  withProps(({ currentUser }) => ({
    canInviteReviewers: get(currentUser, 'permissions.canInviteReviewers'),
    canViewReviewersDetails: get(
      currentUser,
      'permissions.canViewReviewersDetails',
    ),
  })),
)(ReviewerDetails)

// #region styles
const TabButton = styled.div`
  align-items: center;
  border-bottom: ${props =>
    props.selected
      ? `4px solid ${props.theme.colorFurnitureHue}`
      : '4px solid transparent'};
  box-sizing: border-box;
  cursor: pointer;
  display: flex;
  justify-content: center;
  height: calc(${th('gridUnit')} * 5);

  ${marginHelper};

  ${H4} {
    padding: 0 ${th('gridUnit')};
  }
`

const TabsHeader = styled.div`
  align-items: center;
  border-bottom: 1px solid ${th('colorFurniture')};
  background-color: ${th('colorBackgroundHue2')};
  box-sizing: border-box;
  display: flex;
  justify-content: flex-start;

  padding: 0 calc(${th('gridUnit')} * 3);
`

const TabContent = styled.div`
  align-items: stretch;
  background-color: ${th('colorBackgroundHue2')};
  display: flex;
  justify-content: center;
  flex-direction: column;
  min-height: calc(${th('gridUnit')} * 10);
`
// #endregion
